#  ============================================================================
#
#  Copyright (C) 2007-2011 Conceptive Engineering bvba. All rights reserved.
#  www.conceptive.be / project-camelot@conceptive.be
#
#  This file is part of the Camelot Library.
#
#  This file may be used under the terms of the GNU General Public
#  License version 2.0 as published by the Free Software Foundation
#  and appearing in the file license.txt included in the packaging of
#  this file.  Please review this information to ensure GNU
#  General Public Licensing requirements will be met.
#
#  If you are unsure which license is appropriate for your use, please
#  visit www.python-camelot.com or contact project-camelot@conceptive.be
#
#  This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
#  WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#
#  For use of this library in commercial applications, please contact
#  project-camelot@conceptive.be
#
#  ============================================================================

from PyQt4 import QtGui, QtCore

from editors.customeditor import draw_tooltip_visualization

class DecoratedLineEdit(QtGui.QLineEdit):
    """
    A QLineEdit with additional decorations :
    
     * a background text, visible when the line edit doesn't contain any text
     * a validity, which will trigger the background color
    
    Use the user_input method to get the text that was entered by the user. 
    """
      
    arrow_down_key_pressed = QtCore.pyqtSignal()
      
    def __init__(self, parent = None):
        QtGui.QLineEdit.__init__(self, parent)
        self._foreground_color = self.palette().color(self.foregroundRole())
        self._background_color = self.palette().color(self.backgroundRole())
        self._showing_background_text = False
        self._background_text = None
        self._valid = True

    def set_valid(self, valid):
        """Set the validity of the current content of the line edit
        :param valid: True or False
        """
        if valid!=self._valid:
            self._valid = valid
            self._update_background_color()
            
    def set_background_text(self, background_text):
        """Set the text to be displayed in the background when the line
        input does not contain any text
        :param background_text: the text to be shown, None if no text should be shown
        """
        self._hide_background_text()
        self._background_text = background_text
        if not self.hasFocus() and background_text!=None:
            self._show_background_text()
            
    def _show_background_text(self):
        if not self._showing_background_text and not self.text() and self._background_text!=None:
            self._showing_background_text = True
            self._update_foreground_color()
            self.setText(unicode(self._background_text))

    def _hide_background_text(self):
        if self._showing_background_text and self._background_text!=None:
            self._showing_background_text = False
            self._update_foreground_color()
            self.setText('')

    def _update_background_color(self):
        from camelot.view.art import ColorScheme
        palette = self.palette()
        if self._valid:
            palette.setColor(self.backgroundRole(), self._background_color)
        else:
            palette.setColor(self.backgroundRole(), ColorScheme.orange_2)
        self.setPalette(palette)
        
    def _update_foreground_color(self):
        from camelot.view.art import ColorScheme
        palette = self.palette()
        if self._showing_background_text:
            palette.setColor(self.foregroundRole(), ColorScheme.aluminium_1)
        else:
            palette.setColor(self.foregroundRole(), self._foreground_color)
        self.setPalette(palette)
            
    def focusInEvent(self, e):
        self._hide_background_text()
        QtGui.QLineEdit.focusInEvent(self, e)

    def focusOutEvent(self, e):
        self._show_background_text()
        self._update_foreground_color()
        QtGui.QLineEdit.focusOutEvent(self, e)
        
    def keyPressEvent(self, e):
        if e.key() == QtCore.Qt.Key_Down:
            self.arrow_down_key_pressed.emit()
        
        QtGui.QLineEdit.keyPressEvent(self, e)
        
    def user_input(self):
        if self._showing_background_text:
            return u''
        return unicode(self.text())
    
    def set_user_input(self, text):
        if text!=None:
            self._hide_background_text()
            self.setText(text)
        else:
            if not self.hasFocus() and not self._showing_background_text:
                self.setText('')
                self._show_background_text()

    def paintEvent(self, event):
        super(DecoratedLineEdit, self).paintEvent(event)
        
        if self.toolTip():
            draw_tooltip_visualization(self)



#  ============================================================================
#
#  Copyright (C) 2007-2011 Conceptive Engineering bvba. All rights reserved.
#  www.conceptive.be / project-camelot@conceptive.be
#
#  This file is part of the Camelot Library.
#
#  This file may be used under the terms of the GNU General Public
#  License version 2.0 as published by the Free Software Foundation
#  and appearing in the file license.txt included in the packaging of
#  this file.  Please review this information to ensure GNU
#  General Public Licensing requirements will be met.
#
#  If you are unsure which license is appropriate for your use, please
#  visit www.python-camelot.com or contact project-camelot@conceptive.be
#
#  This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
#  WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#
#  For use of this library in commercial applications, please contact
#  project-camelot@conceptive.be
#
#  ============================================================================
"""
Form actions are objects that can be put in the form_actions list of the
object admin interfaces.  Those actions then appear on the form and can
be executed by the end users.  The most convenient method to create custom actions
is to subclass FormAction and implement a custom run method ::

  class MyAction(FormAction):

    def run(self, entity_getter):
      print 'Hello World'

  class Movie(Entity):
    title = Field(Unicode(60), required=True)

    Admin(EntityAdmin):
      list_display = ['title']
      form_actions = [MyAction('Hello')]

Several subclasses of FormAction exist to provide common use cases such as executing
a function in the model thread or printing a report.

To customize the look of the action button on the form, the render method should be
overwritten.
"""
import logging

from PyQt4 import QtGui, QtCore
from camelot.admin.abstract_action import PrintProgressDialog
from camelot.view.art import Icon
from camelot.view.model_thread import gui_function, model_function, post
from camelot.view.controls.progress_dialog import ProgressDialog

logger = logging.getLogger('camelot.admin.form_action')

class FormAction( object ):
    """Abstract base class to implement form actions
    """

    def __init__( self, name, icon = None ):
        """
        :param name: the name used in the button to trigger the action
        :param icon: Icon to be used in the button to trigger the action
        """
        self._name = name
        self._icon = icon

    def get_name(self):
        """:return: the name to be used in the button to trigger the action"""
        return self._name

    def get_icon(self):
        """:return: the Icon to be used in the button to trigger the
action"""
        return self._icon

    def render( self, parent, entity_getter ):
        """:return: a QWidget the user can use to trigger the action, by default
returns a Button that will trigger the run method when clicked"""
        from camelot.view.controls.action_widget import ActionWidget
        return ActionWidget( self, entity_getter, parent=parent )

    def run( self, entity_getter ):
        """Overwrite this method to create an action that does something, the
        run method will be called within the gui thread.

        :param entity_getter: a function that returns the object displayed

        The entity_getter function should not be called within the gui
        thread, it exists for being able to pass it to the model thread.
        """
        raise NotImplementedError

    def enabled(self, entity):
        """Overwrite this method to have the action only enabled for
certain states of the entity displayed.  This method will be called
within the model thread.

:param entity: the entity currently in the form view
:return: True or False, returns True by default
        """
        if entity:
            return True

class FormActionFromGuiFunction( FormAction ):
    """Convert a function that is supposed to run in the GUI thread to a FormAction,
    or"""

    def __init__( self, name, gui_function, icon = None ):
        FormAction.__init__( self, name, icon )
        self._gui_function = gui_function

    @gui_function
    def run( self, entity_getter ):
        self._gui_function( entity_getter )

class FormActionFromModelFunction( FormAction ):
    """Convert a function that is supposed to run in the model thread to a
FormAction.  This type of action can be used to manipulate the model.
    """

    def __init__( self, name, model_function=None, icon = None, flush=False, enabled=lambda obj:True ):
        """
:param name: the name of the action
:param model_function: a function that has 1 arguments, the object
currently in the form, this function will be called whenever the
action is triggered.
:param icon: an Icon
:param flush: flush the object to the db and refresh it in the views, set this to true when the
model function changes the object.
:param enabled: a function that has 1 argument, the object on which the action would be applied
        """
        FormAction.__init__( self, name, icon )
        self._model_function = model_function
        self._flush = flush
        self._enabled = enabled

    @model_function
    def enabled(self, entity):
        """This function will be called in the model thread, to evaluate if the
button should be enabled.

:param entity: the object currently in the form
:return: True or False, defaults to True

        """
        return self._enabled( entity )

    def create_request( self, entity_getter, model_function ):
        """Create the request that will be send to the model thread, this function
        encapsulates the original function in a flush and refresh of the underlying
        entity"""

        def request():
            from sqlalchemy.orm.session import Session
            from camelot.view.remote_signals import get_signal_handler
            o = entity_getter()
            model_function( o )
            if self._flush:
                sh = get_signal_handler()
                Session.object_session( o ).flush( [o] )
                sh.sendEntityUpdate( self, o )
            return True

        return request

    @gui_function
    def run( self, entity_getter ):
        """When the run method is called, a progress dialog will apear while
the model function is executed.

:param entity_getter: a function that when called returns the object currently in the form.
        
        """
        progress = ProgressDialog(self._name)
        post( self.create_request( entity_getter, self._model_function ), progress.finished, exception = progress.exception )
        progress.exec_()

class ProcessFilesFormAction(FormActionFromModelFunction):
    """This action presents the user with a File Selection Dialog, that allows
    the user to select files, those file names are then fed to the process_files
    function.
    
    overwrite the process_files method to have the action do something.
    """

    def create_request( self, entity_getter, model_function ):
        file_names = [unicode(fn) for fn in QtGui.QFileDialog.getOpenFileNames(caption=unicode(self.get_name()))]
        
        def process_files_model_function( obj ):
             return self.process_files( obj, file_names )
                 
        return super(ProcessFilesFormAction, self).create_request( entity_getter, process_files_model_function )
        
    def process_files( self, obj, file_names, options = None ):
        """overwrite this method in a subclass

        :param obj: the object that is displayed in the form
        :param file_names: a list of file names selected by the users
        :param options: an options object if applicable
        """
        pass
    
class PrintHtmlFormAction(FormActionFromModelFunction):
    """Create an action for a form that pops up a print preview for generated html.
Overwrite the html function to customize the html that should be shown::

  class PrintMovieAction(PrintHtmlFormAction):

    def html(self, movie):
      html = '<h1>' + movie.title + '</h1>'
      html += movie.description
    return html

  class Movie(Entity):
    title = Field(Unicode(60), required=True)
    description = Field(camelot.types.RichText)

    class Admin(EntityAdmin):
      list_display = ['title', 'description']
      form_actions = [PrintMovieAction('summary')]

will put a print button on the form :

.. image:: ../_static/formaction/print_html_form_action.png

the rendering of the html can be customised using the HtmlDocument attribute :

.. attribute:: HtmlDocument

the class used to render the html, by default this is
a QTextDocument, but a QtWebKit.QWebView can be used as well.

.. attribute:: PageSize

the page size, the default is QPrinter.A4

.. attribute:: PageOrientation

the page orientation, the default QPrinter.Portrait

.. image:: ../_static/simple_report.png
    """

    HtmlDocument = QtGui.QTextDocument
    PageSize = QtGui.QPrinter.A4
    PageOrientation = QtGui.QPrinter.Portrait

    def __init__(self, name, icon=Icon('tango/16x16/actions/document-print.png')):
        FormActionFromModelFunction.__init__(self, name, self.html, icon)

    def html(self, obj):
        """Overwrite this function to generate custom html to be printed
:param obj: the object that is displayed in the form
:return: a string with the html that should be displayed in a print preview window"""
        return '<h1>' + unicode( obj ) + '<h1>'

    @gui_function
    def run(self, entity_getter):
        progress = PrintProgressDialog(self._name)
        progress.html_document = self.HtmlDocument
        # the progress dialog can easily pass these parameters for us
        progress.page_size = self.PageSize
        progress.page_orientation = self.PageOrientation

        def create_request(entity_getter):

            def request():
                o = entity_getter()
                return self.html(o)

            return request

        post(create_request(entity_getter), progress.print_result, exception=progress.exception)
        progress.exec_()

class OpenFileFormAction( FormActionFromModelFunction ):
    """Form action used to open a file in the prefered application of the user.
To be used for example to generate pdfs with reportlab and open them in
the default pdf viewer.

.. attribute:: suffix

Set the suffix class attribute to the suffix the file should have
eg: .txt or .pdf, defaults to .txt
    """

    suffix = '.txt'

    def __init__( self, name, icon = Icon( 'tango/22x22/actions/document-print.png' ), **kwargs ):

        def model_function( obj ):
            import os
            import tempfile
            file_descriptor, file_name = tempfile.mkstemp(suffix=self.suffix)
            os.close(file_descriptor)
            self.write_file(file_name, obj )
            url = QtCore.QUrl.fromLocalFile(file_name)
            logger.debug(u'open url : %s'%unicode(url))
            QtGui.QDesktopServices.openUrl(url)

        FormActionFromModelFunction.__init__( self, name, model_function, icon, **kwargs )

    def write_file( self, file_name, obj ):
        """
:param file_name: the name of the file to which should be written
:param obj: the object displayed in the form
:return: None

Overwrite this function to generate the file to be opened, this function will be
called when the user triggers the action.  It should write the requested file to the file_name.
This file will then be opened with the system default application for this type of file.
"""
        pass

class ChartFormAction( FormAction ):
    """Action that displays a chart, overwrite its chart
    method.
    """

    def run( self, entity_getter, options=None ):

        def create_request( entity_getter, options ):

            def request():
                o = entity_getter()
                return self.chart( o, options )

            return request

        dialog = ProgressDialog('Rendering Chart')
        post( create_request( entity_getter, options ), dialog.display_chart, dialog.exception )
        dialog.exec_()

    def chart(self, obj, options=None):
        """
        :obj: the object in the form when the user triggered the action
        :options: the options if applicable
        :return: a camelot.container.chartcontainer FigureContainer or AxesContainer object
        """
        from camelot.container.chartcontainer import PlotContainer
        return PlotContainer( [0,1,2,3], [0,2,4,9] )

class PixmapFormAction( FormAction ):
    """Action that displays an image, overwrite its image
    method.
    """

    def run( self, entity_getter, options=None ):

        def create_request( entity_getter, options ):

            def request():
                o = entity_getter()
                return self.pixmap( o, options )

            return request

        dialog = ProgressDialog('Rendering Image')
        post( create_request( entity_getter, options ), 
              dialog.display_pixmap, 
              dialog.exception )
        dialog.exec_()

    def pixmap(self, obj, options=None):
        """
        :obj: the object in the form when the user triggered the action
        :options: the options if applicable
        :return: a camelot.view.art.Pixmap object
        """
        return None
    
class DocxFormAction( FormActionFromModelFunction ):
    """Action that generates a .docx file and opens it using Word.  It does so by generating an xml document
with jinja templates that is a valid word document.  Implement at least its get_template method in a subclass
to make this action functional.
    """

    def __init__( self, name, icon = Icon( 'tango/16x16/mimetypes/x-office-document.png' ) ):
        FormActionFromModelFunction.__init__( self, name, self.open_xml, icon )

    def get_context(self, obj):
        """
:param obj: the object displayed in the form
:return: a dictionary with objects to be used as context when jinja fills up the xml document,
by default returns a context that contains obj"""
        return {'obj':obj}

    def get_environment(self, obj):
        """
:param obj: the object displayed in the form
:return: the jinja environment to be used to render the xml document, by default returns an
empty environment"""
        from jinja2 import Environment
        e = Environment()
        return e

    def get_template(self, obj):
        """
:param obj: the object displayed in the form
:return: the name of the jinja template for xml document.  A template can be constructed by
creating a document in MS Word and saving it as an xml file.  This file can then be manipulated by hand
to include jinja constructs."""
        raise NotImplemented

    def document(self, obj):
        """
:param obj: the object displayed in the form
:return: the xml content of the generated document. This method calls get_environment,
get_template and get_context to create the final document."""
        e = self.get_environment(obj)
        context = self.get_context(obj)
        t = e.get_template(self.get_template(obj))
        document_xml = t.render(context)
        return document_xml

    def open_xml(self, obj):
        from camelot.view.export.word import open_document_in_word
        import tempfile
        import os
        fd, fn = tempfile.mkstemp(suffix='.xml')
        docx_file = os.fdopen(fd, 'wb')
        docx_file.write(self.document(obj).encode('utf-8'))
        docx_file.close()
        open_document_in_word(fn)

def structure_to_form_actions( structure ):
    """Convert a list of python objects to a list of form actions.  If the python
    object is a tuple, a FormActionFromGuiFunction is constructed with this tuple as arguments.  If
    the python object is an instance of a FormAction, it is kept as is.
    """

    def object_to_action( o ):
        if isinstance( o, FormAction ):
            return o
        return FormActionFromGuiFunction( o[0], o[1] )

    return [object_to_action( o ) for o in structure]





#  ============================================================================
#
#  Copyright (C) 2007-2011 Conceptive Engineering bvba. All rights reserved.
#  www.conceptive.be / project-camelot@conceptive.be
#
#  This file is part of the Camelot Library.
#
#  This file may be used under the terms of the GNU General Public
#  License version 2.0 as published by the Free Software Foundation
#  and appearing in the file license.txt included in the packaging of
#  this file.  Please review this information to ensure GNU
#  General Public Licensing requirements will be met.
#
#  If you are unsure which license is appropriate for your use, please
#  visit www.python-camelot.com or contact project-camelot@conceptive.be
#
#  This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
#  WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#
#  For use of this library in commercial applications, please contact
#  project-camelot@conceptive.be
#
#  ============================================================================

from fileeditor import FileEditor
from fileeditor import CustomEditor
from wideeditor import WideEditor

from camelot.view.art import Icon
from camelot.core.utils import ugettext_lazy as _
from camelot.view.controls.liteboxview import LiteBoxView
from camelot.view.model_thread import post
from camelot.view.action import ActionFactory

from PyQt4 import QtGui
from PyQt4 import QtCore
from PyQt4.QtCore import Qt

class ImageEditor(FileEditor, WideEditor):
    """Editor to view and edit image files, this is a customized
    implementation of a FileEditor"""

    filter = """Image files (*.bmp *.jpg *.jpeg *.mng *.png *.pbm *.pgm *.ppm
*.tiff *.xbm *.xpm) All files (*)"""

    def __init__(
            self,
            parent=None,
            storage=None,
            preview_width=100,
            preview_height=100,
            **kwargs
        ):
        self.preview_width = preview_width
        self.preview_height = preview_height
        FileEditor.__init__(
            self, parent=parent, storage=storage, **kwargs
        )

    def setup_widget(self):
        self.layout = QtGui.QHBoxLayout()
        self.layout.setContentsMargins( 0, 0, 0, 0 )
        #
        # Setup label
        #
        self.label = QtGui.QLabel(self)
        self.label.installEventFilter(self)
        self.layout.addWidget(self.label)
        self.label.setAlignment(Qt.AlignHCenter|Qt.AlignVCenter)
        #
        # Setup buttons
        #
        button_layout = QtGui.QVBoxLayout()
        button_layout.setSpacing(0)
        button_layout.setMargin(0)

        self.open_button = QtGui.QToolButton()
        self.open_button.setIcon(self.open_icon)
        self.open_button.setAutoRaise(True)
        self.open_button.setToolTip(unicode(_('open image')))
        self.open_button.clicked.connect(self.open_button_clicked)

        self.clear_button = QtGui.QToolButton()
        self.clear_button.setIcon(self.clear_icon)
        self.clear_button.setToolTip(unicode(_('delete image')))
        self.clear_button.setAutoRaise(True)
        self.clear_button.clicked.connect(self.clear_button_clicked)
        
        copy_button = QtGui.QToolButton()
        copy_button.setDefaultAction( ActionFactory.copy(self, self.copy_to_clipboard ) )
        copy_button.setAutoRaise(True)

        paste_button = QtGui.QToolButton()
        paste_button.setDefaultAction( ActionFactory.paste(self, self.paste_from_clipboard ) )
        paste_button.setAutoRaise(True)
        paste_button.setObjectName('paste')
        
        button_layout.addStretch()
        button_layout.addWidget(self.open_button)
        button_layout.addWidget(self.clear_button)
        button_layout.addWidget(copy_button)
        button_layout.addWidget(paste_button)

        self.layout.addLayout(button_layout)
        self.layout.addStretch()
        self.setLayout(self.layout)
        self.clear_image()
        QtGui.QApplication.clipboard().dataChanged.connect( self.clipboard_data_changed )
        self.clipboard_data_changed()
        
    @QtCore.pyqtSlot()
    def clipboard_data_changed(self):
        paste_button = self.findChild(QtGui.QWidget, 'paste')
        if paste_button:
            mime_data = QtGui.QApplication.clipboard().mimeData()
            paste_button.setVisible( mime_data.hasImage() )
            
    @QtCore.pyqtSlot()
    def paste_from_clipboard(self):
        """Paste an image from the clipboard into the editor"""
        mime_data = QtGui.QApplication.clipboard().mimeData()
        if mime_data.hasImage():
            byte_array = QtCore.QByteArray()
            buffer = QtCore.QBuffer( byte_array )
            image = QtGui.QImage( mime_data.imageData() )
            image.save( buffer, 'PNG' )
            
            def create_checkin( byte_array ):
                return lambda:self.checkin_byte_array(byte_array, '.png')
            
            post( create_checkin( byte_array ), self.stored_file_ready )
        
    def checkin_byte_array(self, byte_array, suffix):
        """Check a byte_array into the storage"""
        import cStringIO
        stream = cStringIO.StringIO( byte_array.data() )
        return self.storage.checkin_stream( 'clipboard', suffix, stream)
        
    def set_enabled(self, editable=True):
        self.clear_button.setEnabled(editable)
        self.open_button.setEnabled(editable)
        self.label.setEnabled(editable)

    def set_pixmap(self, pixmap):
        self.label.setPixmap(pixmap)
        self.draw_border()

    def set_image(self, image):
        self.set_pixmap(QtGui.QPixmap.fromImage(image))
        
    @QtCore.pyqtSlot()
    def copy_to_clipboard(self):
        """Copy the image to the clipboard"""
        if self.value:
            post( self.value.checkout_image, self.set_image_to_clipboard )
        
    def set_image_to_clipboard(self, image):
        clipboard = QtGui.QApplication.clipboard()
        clipboard.setImage( image )

    def clear_image(self):
        dummy_image = Icon('tango/32x32/mimetypes/image-x-generic.png')
        self.set_pixmap(dummy_image.getQPixmap())

    def set_value(self, value):
        value = CustomEditor.set_value(self, value)
        if value:
            self.open_button.setIcon(self.open_icon)
            self.open_button.setToolTip(unicode(_('open file')))
            if value!=self.value:
                post(
                    lambda:value.checkout_thumbnail(
                        self.preview_width,
                        self.preview_height
                    ),
                    self.set_image
                )
        else:
            self.clear_image()
            self.open_button.setIcon(self.new_icon)
            self.open_button.setToolTip(unicode(_('add file')))
        self.value = value
        return value

    def draw_border(self):
        self.label.setFrameShape(QtGui.QFrame.Box)
        self.label.setFrameShadow(QtGui.QFrame.Plain)
        self.label.setLineWidth(1)
        self.label.setFixedSize(self.preview_width, self.preview_height)

    def show_fullscreen(self, image):
        lite_box = LiteBoxView(self)
        lite_box.show_fullscreen_image(image)

    def eventFilter(self, object, event):
        if not object.isWidgetType():
            return False
        if event.type() != QtCore.QEvent.MouseButtonPress:
            return False
        if event.modifiers() != QtCore.Qt.NoModifier:
            return False
        if event.buttons() == QtCore.Qt.LeftButton:
            if self.value:
                post(
                    lambda:self.value.checkout_thumbnail(640,480),
                    self.show_fullscreen
                )
            return True
        return False





from camelot.admin.object_admin import ObjectAdmin
from camelot.view.controls import delegates

def dynamic_tooltip_x(coordinate):
  return u'The <b>x</b> value of the coordinate, now set to %s'%(coordinate.x)

def dynamic_tooltip_y(coordinate):
  return u'The <b>y</b> value of the coordinate, now set to %s'%(coordinate.y)
  
class Coordinate(object):
  
  def __init__(self):
    self.id = 1
    self.x = 0.0
    self.y = 0.0
      
  class Admin(ObjectAdmin):
    form_display = ['x', 'y',]
    field_attributes = dict(x=dict(delegate=delegates.FloatDelegate, 
                                   tooltip=dynamic_tooltip_x),
                            y=dict(delegate=delegates.FloatDelegate,
                                   tooltip=dynamic_tooltip_y),
                            )
    form_size = (100,100)


#  ============================================================================
#
#  Copyright (C) 2007-2011 Conceptive Engineering bvba. All rights reserved.
#  www.conceptive.be / project-camelot@conceptive.be
#
#  This file is part of the Camelot Library.
#
#  This file may be used under the terms of the GNU General Public
#  License version 2.0 as published by the Free Software Foundation
#  and appearing in the file license.txt included in the packaging of
#  this file.  Please review this information to ensure GNU
#  General Public Licensing requirements will be met.
#
#  If you are unsure which license is appropriate for your use, please
#  visit www.python-camelot.com or contact project-camelot@conceptive.be
#
#  This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
#  WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#
#  For use of this library in commercial applications, please contact
#  project-camelot@conceptive.be
#
#  ============================================================================

"""left navigation pane"""

import logging
logger = logging.getLogger('camelot.view.controls.navpane3')

from PyQt4 import QtGui, QtCore

from PyQt4.QtCore import Qt

from PyQt4.QtGui import QMenu, QFrame, QWidget, \
    QDockWidget, QVBoxLayout, QHBoxLayout

from camelot.view.action import ActionFactory
from camelot.view.model_thread import post
from camelot.view.controls.modeltree import ModelItem, ModelTree

class PaneButton(QWidget):
    """Custom made navigation pane button"""

    width = 160
    height = 32

    pressed = QtCore.pyqtSignal(int)

    def __init__(self, text, icon_pixmap, parent=None):
        super(PaneButton, self).__init__(parent)

        button = QtGui.QPushButton(QtGui.QIcon(icon_pixmap), unicode(text), self)
        button.pressed.connect( self._button_pressed )
        #button.setFlat(True)
        layout = QHBoxLayout()
        layout.setSpacing(0)
        layout.setContentsMargins(0,0,0,0)

        #self.icon = QLabel(self)
        #self.icon.setSizePolicy(QSizePolicy(
        #    QSizePolicy.Fixed, QSizePolicy.Preferred))
        #self.icon.setPixmap(icon_pixmap)
        #self.icon.setObjectName('PaneButtonIcon')
        #layout.addWidget(self.icon)

        #self.label = UserTranslatableLabel(text, self)
        #self.label.setAlignment(Qt.AlignLeft|Qt.AlignVCenter)
        #self.label.setObjectName('PaneButtonLabel')
        layout.addWidget( button )

        self.setLayout(layout)

        self.setFixedHeight(self.height)
        self.setMinimumWidth(self.width)
        self.resize(self.width, self.height)

        self.setObjectName('PaneButton')

    def toggle_bold(self):
        pass
        #font = self.label.font()
        #font.setBold(not font.bold())
        #self.label.setFont(font)

    def layout_index(self):
        return self.parent().layout().indexOf(self)

    @QtCore.pyqtSlot()
    def _button_pressed(self):
        self.pressed.emit(self.layout_index())

class NavigationPane(QDockWidget):
    """NavigationPane made of PaneButtons and ModelTrees"""

    def __init__(self, app_admin, workspace, parent):
        super(QDockWidget, self).__init__(parent)

        self.workspace = workspace
        self.app_admin = app_admin
        self._sections = None
        self._animation = QtCore.QParallelAnimationGroup()
        self._animation.finished.connect( self.animation_finished )
        
        self._title_widget = QWidget()

        self._dock_widget = self.get_dock_widget()
        self._dock_widget.setMouseTracking(True)

        self._current_tree_widget = None

        # hack for removing the dock title bar
        self.setTitleBarWidget(self._title_widget)
        self.setWidget(self._dock_widget)

        self.setFeatures(QDockWidget.NoDockWidgetFeatures)

        self.app_admin.sections_changed_signal.connect(self.update_sections)
        self.update_sections()

    @QtCore.pyqtSlot()
    def update_sections(self):
        post(self.app_admin.get_sections, self.set_sections)
        
    @QtCore.pyqtSlot()
    def animation_finished(self):
        """Once the animation is finished, hide all non active
        section trees"""
        layout = self._dock_widget.layout()
        for i in range( layout.count() ):
            if i%2 == 1:
                section_tree = layout.itemAt( i ).widget()
                if section_tree != self._current_tree_widget:
                    section_tree.hide()
        
    def get_dock_widget(self):
        layout = QVBoxLayout()
        layout.setContentsMargins(0,0,0,0)
        layout.setSpacing(0)

        dw = QWidget()
        dw.setLayout(layout)
        return dw

    def get_tree_widget(self):
        tw = ModelTree(parent=self)
        tw.setObjectName('PaneTree')

        # i hate the sunken frame style
        tw.setFrameShape(QFrame.NoFrame)
        tw.setFrameShadow(QFrame.Plain)

        tw.contextmenu = QMenu(self)
        act = ActionFactory.new_tab(self, self.open_in_new_view)
        tw.contextmenu.addAction(act)
        tw.setContextMenuPolicy(Qt.CustomContextMenu)
        tw.customContextMenuRequested.connect(self.create_context_menu)

        return tw

    def get_sections(self):
        return self._sections

    def set_sections(self, sections):
        logger.debug('setting navpane sections')

        animation = QtCore.QPropertyAnimation(self._dock_widget, 'minimumWidth', self)
        animation.setDuration( 500 )
        animation.setStartValue( 10 )
        animation.setEndValue( 220 )
        animation.start()
        
        self._sections = sections
        self._buttons = [(
            index,
            section.get_verbose_name(),
            section.get_icon().getQPixmap(),
        ) for index, section in enumerate(sections)]

        for index, name, pixmap in self._buttons:
            tree_widget = self.get_tree_widget()
            if index!=0:
                tree_widget.setMaximumHeight(0)

            pane_button = PaneButton(name, pixmap)
            pane_button.pressed.connect(self.change_current)

            self._dock_widget.layout().addWidget(pane_button)
            self._dock_widget.layout().addWidget(tree_widget)

            last_tree_index = self._dock_widget.layout().count() - 1

            def get_models_for_tree():
                return (last_tree_index, self._sections[index].get_items())

            post(get_models_for_tree, self.set_models_for_tree)

        self.change_current(0)

    def set_models_for_tree(self, (tree_index, items)):
        tree_widget = self._dock_widget.layout().itemAt(tree_index).widget()
        for item in items:
            label = item.get_verbose_name()
            tree_widget.section_items.append(item)
            tree_widget.modelitems.append(ModelItem(tree_widget, [label]))
            tree_widget.itemClicked.connect(self.open_in_current_view)

    @QtCore.pyqtSlot(int)
    def change_current(self, index):
        logger.debug('setting current navpane index to %s' % index)

        self._animation.stop()
        self._animation.clear()
        
        if self._current_tree_widget is not None:
            current_index = self._dock_widget.layout().indexOf(self._current_tree_widget) - 1
            if index == current_index:
                return
            self._dock_widget.layout().itemAt(current_index).widget().toggle_bold()
            hide = QtCore.QPropertyAnimation(self._current_tree_widget, 'maximumHeight')
            hide.setDuration( 150 )
            hide.setStartValue( self._current_tree_widget.height() )
            hide.setEndValue( 0 )
            self._animation.addAnimation( hide )
                    
        self._dock_widget.layout().itemAt(index).widget().toggle_bold()
        tree_widget = self._dock_widget.layout().itemAt(index+1).widget()
        tree_widget.show()
        show = QtCore.QPropertyAnimation(tree_widget, 'maximumHeight')
        show.setDuration( 150 )
        show.setStartValue( 0 )
        show.setEndValue( 1000 )
        self._animation.addAnimation( show )
        self._animation.start()
        self._current_tree_widget = tree_widget

    def get_section_item(self, item):
        index = self._current_tree_widget.indexFromItem(item)
        return self._current_tree_widget.section_items[index.row()]

    def create_context_menu(self, point):
        logger.debug('creating context menu')
        item = self._current_tree_widget.itemAt(point)
        if item:
            self._current_tree_widget.setCurrentItem(item)
            self._current_tree_widget.contextmenu.popup(
                self._current_tree_widget.mapToGlobal(point)
            )

    @QtCore.pyqtSlot(QtGui.QTreeWidgetItem, int)
    def open_in_current_view(self, item, _column):
        """pops a model window in parent's workspace"""
        logger.debug('poping a window in parent')
        item = self._current_tree_widget.currentItem()
        index = self._current_tree_widget.indexFromItem(item)
        section_item = self._current_tree_widget.section_items[index.row()]
        new_view = section_item.get_action().run(self.workspace)
        if new_view:
            self.workspace.set_view(new_view)

    @QtCore.pyqtSlot()
    def open_in_new_view(self):
        """pops a model window in parent's workspace"""
        logger.debug('poping a window in parent')
        item = self._current_tree_widget.currentItem()
        index = self._current_tree_widget.indexFromItem(item)
        section_item = self._current_tree_widget.section_items[index.row()]
        new_view = section_item.get_action().run(self.workspace)
        if new_view:
            self.workspace.add_view(new_view)




#  ============================================================================
#
#  Copyright (C) 2007-2011 Conceptive Engineering bvba. All rights reserved.
#  www.conceptive.be / project-camelot@conceptive.be
#
#  This file is part of the Camelot Library.
#
#  This file may be used under the terms of the GNU General Public
#  License version 2.0 as published by the Free Software Foundation
#  and appearing in the file license.txt included in the packaging of
#  this file.  Please review this information to ensure GNU
#  General Public Licensing requirements will be met.
#
#  If you are unsure which license is appropriate for your use, please
#  visit www.python-camelot.com or contact project-camelot@conceptive.be
#
#  This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
#  WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#
#  For use of this library in commercial applications, please contact
#  project-camelot@conceptive.be
#
#  ============================================================================

"""Proxy objects for the model that live in the GUI thread and are able to
respond immediately. They do so by keeping a local copy of the data of the real
model

We take copies of the data in the elixir model, because accessing an attribute
on the model, might cause an sql query which might take a long time, and might
freeze the gui
"""

class ValueLoading(object):
    """Class indicating that a value was not yet loaded into the
    proxy"""
    pass


